#ifndef DHT11_H
#define	DHT11_H
//------------------------------------------------------------------------------
#include "DS18B20_pins.h"
#define FRAC_FLAG           0x000F
#define SKIP_ROM            0xCC
#define START_CONVERSION    0x44
#define READ_TEMPERATURE    0xBE
//------------------------------------------------------------------------------ 
__bit ds18b20_start()
{
  DQ_PIN = 0;           // send reset pulse to the DS18B20 sensor
  DQ_PIN_Dir = 0;       // configure DQ_PIN pin as output
  __delay_us(500);      
 
  DQ_PIN_Dir = 1;       // configure DQ_PIN pin as input
  __delay_us(100);      // wait 100 us to read the DS18B20 sensor response
 
  if(!DQ_PIN)
  {
    __delay_us(400);    // wait 400 us
    return 1;           // DS18B20 sensor is present
  }
 
  return 0;   // connection error
}
//------------------------------------------------------------------------------ 
void ds18b20_writeBit(char data)
{
  DQ_PIN = 0;
  DQ_PIN_Dir = 0;               // configure DQ_PIN pin as output
  __delay_us(2);       
 
  DQ_PIN = (__bit) data;
  __delay_us(80);       
 
  DQ_PIN_Dir = 1;               // configure DQ_PIN pin as input
  __delay_us(2);        
}
//------------------------------------------------------------------------------ 
void ds18b20_writeByte(char data)
{
  for(char i = 0; i < 8; i++)
  {
    ds18b20_writeBit(data >> i);
  }
}
//------------------------------------------------------------------------------ 
__bit ds18b20_readBit()
{
  static __bit data;
 
  DQ_PIN = 0;
  DQ_PIN_Dir = 0;             // configure DQ_PIN pin as output
  __delay_us(2);
 
  DQ_PIN_Dir = 1;           // configure DQ_PIN pin as input
  __delay_us(5);
 
  data = DQ_PIN;            // read and store DS18B20 state
  __delay_us(100);      
 
  return data;
}
//------------------------------------------------------------------------------ 
char ds18b20_readByte()
{
  char data = 0;
 
  for(char i = 0; i < 8; i++)
  {
    data += ds18b20_readBit() << i;
  }
  
  return data;
}
//------------------------------------------------------------------------------
__bit ds18b20_command(char data)
{
  if(!ds18b20_start())
  {
    return 0;             // return 0 if error
  }
    
  ds18b20_writeByte(SKIP_ROM);   
  ds18b20_writeByte(data);       
}
//------------------------------------------------------------------------------
__bit ds18b20_read(signed int *rawTempr)
{
  
  if(!ds18b20_command(START_CONVERSION))
  {
    return 0;
  }
 
  while(ds18b20_readByte() == 0);  // wait for conversion complete
 
  if(!ds18b20_command(READ_TEMPERATURE))
  {
    return 0;
  }
 
  *rawTempr += (ds18b20_readByte() << 8) + ds18b20_readByte();
 
  return 1;                        // OK --> return 1                  
}
//------------------------------------------------------------------------------
float ds18b20_toFloat(signed int tempr)
{
  float result = (float)(tempr>>4);     
  char count = tempr & FRAC_FLAG;          
  
  if(count) 
  {
    result += (count * 0.0625);
  }
  
  return result;
}
//------------------------------------------------------------------------------
#endif	/* DHT11_H */